/*******************************************************************************
 * Copyright (c) 2003-2007 Network Analysis Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.calculation;

import java.util.ArrayList;
import java.util.HashMap;

import org.StringManipulationTools;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.TableData;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.kegg_expression.KeggExpressionDatapoint;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.dbe.kegg_expression.KeggExpressionDataset;

public class KeggExpressionConverterEva {

	ArrayList<KeggExpressionDataset> datasets;

	HashMap<String, String> filename2organism = new HashMap<String, String>();
	HashMap<String, String> filename2organismId = new HashMap<String, String>();
	HashMap<String, String> filename2time = new HashMap<String, String>();
	HashMap<String, String> filename2replicate = new HashMap<String, String>();

	String expName = "expression analysis";
	String coordinator = "experiment coordinator";
	String startOfExp = "";
	String timeUnit = "-1";
	String measurementUnit = "expression";

	public KeggExpressionConverterEva(ArrayList<KeggExpressionDataset> datasets) {
		this.datasets = datasets;
	}

	public String getDesiredExperimentName() {
		return expName;
	}

	public String getDesiredCoordinatorValue() {
		return coordinator;
	}

	public String getDesiredTimeUnit() {
		return timeUnit;
	}

	public String getDesiredMeasurementUnit() {
		return measurementUnit;
	}

	public void getDescriptionDataFromUser() {
		// ask user for:
		// experiment name
		// coordinator
		// start of experiment
		// time unit
		// 
		// fileName | Plant-ID | Time-Point | Replicate ID
		// ------------------------------------------------
		// (no edit) | ? | ? | ?
		//
		// evaluate datasets for fileName, and ask for remaining field data

		for (KeggExpressionDataset ked : datasets) {
			String filename = ked.getFileName();

			if (filename == null)
				filename = "";
			String on = ked.getOrganismName();

			if (on == null)
				on = "";
			String organismId = on + "";

			String defaultTime = "-1";
			try {
				filename = StringManipulationTools.stringReplace(filename, "\"", "");
				filename = StringManipulationTools.stringReplace(filename, "\'", "");
				if (filename.endsWith(".0"))
					filename = filename.substring(0, filename.length() - ".0".length());
				defaultTime = Integer.parseInt(filename) + "";
			} catch (Exception e) {
				defaultTime = "-1";
			}
			String gt = "genotype";
			if (defaultTime.equalsIgnoreCase("-1"))
				gt = filename;
			String organismInput = gt;
			String timePointInput = defaultTime;
			String replicateInput = "1";
			filename2organismId.put(filename, organismId);
			filename2organism.put(filename, organismInput);
			filename2time.put(filename, timePointInput);
			filename2replicate.put(filename, replicateInput);

		}
	}

	public TableData getDatasetTable() {
		TableData tab = new TableData();
		int row = 1;
		tab.addCellData(1 - 1, row - 1, "clust-ID"); // gene identifier column header
		tab.addCellData(2 - 1, row - 1, "info");
		row++;
		int col = 3;
		HashMap<String, Integer> knownGene2row = new HashMap<String, Integer>();
		for (KeggExpressionDataset ked : datasets) {
			String filename = ked.getFileName();
			if (filename2organismId.get(filename) == null)
				continue;
			String organism = filename2organismId.get(filename);
			String plant = filename2organism.get(filename);
			String time = filename2time.get(filename);
			String repl = filename2replicate.get(filename);
			if (ked.isTrueKeggExpressionFormatControlTarget()) {
				String columnHeaderA = time + plant + "-control" + "_" + repl;
				String columnHeaderB = time + plant + "-target" + "_" + repl;
				tab.addCellData(col - 1, 1 - 1, columnHeaderA);
				tab.addCellData(col + 1 - 1, 1 - 1, columnHeaderB);
			} else {
				String columnHeader = time + plant + "_" + repl;
				tab.addCellData(col - 1, 1 - 1, columnHeader);
			}
			String orgPrefix;
			if (organism != null && organism.length() > 0)
				orgPrefix = organism + ":";
			else
				orgPrefix = "";
			for (KeggExpressionDatapoint dp : ked.getDataPoints()) {
				String activeGeneId = orgPrefix + dp.getGeneId();
				if (dp.getGeneId().indexOf("^") >= 0) {
					activeGeneId = StringManipulationTools.stringReplace(activeGeneId, "^", "^" + orgPrefix);
				}
				if (knownGene2row.containsKey(activeGeneId)) {
					int knownRow = knownGene2row.get(activeGeneId);
					if (ked.isTrueKeggExpressionFormatControlTarget())
						tab.addCellData(col - 1, knownRow - 1, dp.getC());
					else
						tab.addCellData(col - 1, knownRow - 1, dp.getC() + dp.getOptQualityTag(":", ""));

					if (ked.isTrueKeggExpressionFormatControlTarget())
						tab.addCellData(col + 1 - 1, knownRow - 1, dp.getT());
				} else {
					knownGene2row.put(activeGeneId, row);
					tab.addCellData(1 - 1, row - 1, activeGeneId);
					if (dp.getOptX() != null && dp.getOptY() != null)
						tab.addCellData(2 - 1, row - 1, dp.getOptX() + ":" + dp.getOptY());

					if (ked.isTrueKeggExpressionFormatControlTarget())
						tab.addCellData(col - 1, row - 1, dp.getC());
					else
						tab.addCellData(col - 1, row - 1, dp.getC() + dp.getOptQualityTag(":", ""));

					if (ked.isTrueKeggExpressionFormatControlTarget())
						tab.addCellData(col + 1 - 1, row - 1, dp.getT());
					row++;
				}
			}
			col++;
			if (ked.isTrueKeggExpressionFormatControlTarget())
				col++;
		}
		return tab;
	}
}
