/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.calculation;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.TreeSet;

import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.editor.GravistoService;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MessageType;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.BooleanParameter;
import org.graffiti.plugin.parameter.ObjectListParameter;
import org.graffiti.plugin.parameter.Parameter;
import org.graffiti.plugin.parameter.SelectionParameter;
import org.graffiti.selection.Selection;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.editcomponents.chart_settings.GraffitiCharts;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.dbe.algorithms.RemoveMappingDataAlgorithm;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ExperimentInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.helper_classes.Experiment2GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.helper_classes.MapResult;
import de.ipk_gatersleben.ag_nw.graffiti.services.task.BackgroundTaskStatusProviderSupportingExternalCallImpl;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.ExecutionParameter;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBASimVizAddon;
import de.ipk_gatersleben.ag_pbi.fluxbalance.visualization.AnimatedFluxVisualizationDeletion;

/**
 * @author Eva
 */
public class StartCalculationAlgorithmDeletionNew extends AbstractStartCalculationAlgorithm {
	
	private static String ofReaction;
	
	private static boolean maximization = true;
	// private boolean minimization = false;
	private static boolean linearOpt = false;
	private static boolean singleDel = true;
	// private boolean multipleDel = false;
	private int del;
	private String delReactions;
	private boolean check = true;
	private static Selection delReactionSelection;
	// private final boolean additionalConstraints = true;
	// private boolean SBMLfileGeneration = true;
	private int ofCoef;
	private int noRcts;
	private double computationTime;
	private String nlopt;
	
	public String getName() {
		return "<html><font color=\"#0101DF\">Knock-out Analysis (specified)</font>";
	}
	
	@Override
	public String getDescription() {
		return "<html>" +
							"<b>With this command you may perform a knock-out analysis<br>" +
							"based on selected reactions specified by the user</b><br>" +
							"<br>" +
							"To perform the analysis you have to specify:<br>" +
							"(1) the list of reactions to be knocked-out by selecting<br>" +
							"the corresponding nodes in the graph<br>" +
							"(2) the objective function (Reaction to be optimized)<br>" +
							"(3) the mode of optimization (Maximization, Minimization)<br>" +
							"(4) the type of optimization (Linear, Non-linear)<br>" +
							"<br>";
	}
	
	@Override
	public Parameter[] getParameters() {
		
		TreeSet<String> reactionNames = FBAHelper.getReactionNamesUnique(graph.getNodes());
		
		
		if (ofReaction == null)
            ofReaction = reactionNames.iterator().next(); 
				
		
		
		return new Parameter[] {
				delReactionSelection == null ? new SelectionParameter("Reactions to be knocked-out", "User-specified reactions which will be deleted") :
						new SelectionParameter(delReactionSelection, "Reactions to be knocked-out", "User-specified reactions which will be deleted"),
							new BooleanParameter(
									singleDel,
												"Single Knock-out",
												"Activated checkbox: sequential deletion of specified reactions; not activated checkbox: multiple deletion of specified reactions"),
							// new BooleanParameter(false, "Multiple Knock-out", "Falls gew�hlt, werden die ausgew�hlten Reaktionen gemeinsam ausgeknockt"),
				new ObjectListParameter(reactionNames.contains(ofReaction) ? ofReaction : reactionNames.iterator().next(), "Reaction to be optimized",
						"Objective function", reactionNames),
							new BooleanParameter(maximization, "Maximization", "Activated checkbox: maximization; not activated checkbox: minimization"),
							// new BooleanParameter(false, "Minimization", "Falls gew�hlt, wird die OF minimiert...OFcoef = 1"),
				new BooleanParameter(linearOpt, "Linear Optimization",
												"Activated checkbox: linear optimisation; not activated checkbox: nonlinear optimisation (2-step optimisation procedure)"),
		// new BooleanParameter(false, "Account for changed reaction bounds", "Falls gew�hlt, werden die ge�nderten constraints ber�cksichtigt..."),
		// new BooleanParameter(false, "Automatic SBML-file generation", "Falls gewählt, wird Berechnung auf neu generiertem SBML-file durchgeführt"),
		
		};
	}
	
	@Override
	public void setParameters(Parameter[] params) {
		
		int i = 0;
		delReactionSelection = ((SelectionParameter) params[i++]).getSelection();
		System.out.println("delReaction: "+delReactionSelection);
		singleDel = ((BooleanParameter) params[i++]).getBoolean();
		// this.multipleDel = ((BooleanParameter)params[i++]).getBoolean();
		if (singleDel) {
			del = 1;
		} else {
			del = 2;
		}
		ofReaction = (String) ((ObjectListParameter) params[i++]).getValue();
		maximization = ((BooleanParameter) params[i++]).getBoolean();
		// this.minimization = ((BooleanParameter)params[i++]).getBoolean();
		if (maximization) {
			ofCoef = -1;
		} else {
			ofCoef = 1;
		}
		linearOpt = ((BooleanParameter) params[i++]).getBoolean();
		if (linearOpt) {
			nlopt = "false";
		} else {
			nlopt = "true";
		}
		// this.additionalConstraints = ((BooleanParameter)params[i++]).getBoolean();
		// this.SBMLfileGeneration = ((BooleanParameter)params[i++]).getBoolean();
		
		delReactions = getDelReactionsString(delReactionSelection);
		
		// Eva: 9.11.
		if (delReactions.indexOf(ofReaction) != -1) {
			check = getDelCheck(delReactionSelection, ofReaction);
			// check = false;
		}
	}
	
	@Override
	public void check() throws PreconditionException {
		super.check();
		PreconditionException errors = new PreconditionException();
		
		if (selection.getNumberOfNodes() <= 0) {
			errors
					.add("To perform Knock-out Analysis, please select first the reactions to be deleted by clicking on the corresponding reaction nodes in the graph!");
		}
		/*
		 * if (singleDel == true && multipleDel == true){
		 * errors.add("Please chose one mode of Knock-out analysis (Single or Multiple).");
		 * this.singleDel = true;
		 * this.multipleDel = false;
		 * }
		 */

		checkForMatlabExecutable(errors, "VdelSpecJan.exe");
		
		/*
		 * if (maximization == true && minimization == true){
		 * errors.add("Please chose one mode of optimization: maximization or minimization.");
		 * this.maximization = true;
		 * this.minimization = false;
		 * }
		 * if (maximization == false && minimization == false){
		 * errors.add("Please specify the mode of optimization: maximization or minimization.");
		 * this.maximization = true;
		 * this.minimization = false;
		 * }
		 */

		if (!errors.isEmpty()) {
			throw errors;
		}
	}
	
	public void execute() {
		
		final ExecutionParameter par = FBAHelper.prepareExecution(graph, true);
		if (par == null)
			return;
		
		doExecute(graph, par);
	}
	
	public void doExecute(final Graph graph, final ExecutionParameter par) {
		
		if (check == false) {
			MainFrame.showMessageDialog("The reaction to be knocked-out has to be different from the reaction to be optimized.", "Execution Aborted");
			ofReaction = null;
			check = true;
		}
		
		final BackgroundTaskStatusProviderSupportingExternalCallImpl status =
				new BackgroundTaskStatusProviderSupportingExternalCallImpl("Zeile 1", "Zeile 2");
		
		FBAHelper.runCalculation(status, getName(), new Runnable() {
			@Override
			public void run() {
				try {
					
					String SBMLmodel = par.filenameSBML;
					String OFrct = ofReaction;
					String OFcoef = (new Integer(ofCoef).toString());
					String DelMode = (new Integer(del).toString());
					String DELrct = delReactions;
					String opt = nlopt;
					
					// exe-file und input/output files muessen im gleichen Ordener sein
					List<String> command = new ArrayList<String>();
					
					// String pathExe = ReleaseInfo.getAppSubdirFolderWithFinalSep("addons") + "VdelStartNEW.exe";
					String pathExe = FBASimVizAddon.pathExeAddons + "VdelSpecJan.exe";
					
					command.add(pathExe);
					command.add(SBMLmodel);
					command.add(opt);
					command.add(OFrct);
					command.add(OFcoef);
					command.add("NM.txt");
					command.add("VM.txt");
					command.add(DELrct);
					command.add(DelMode);
					
					if (par.additionalConstraints == true) {
						command.add("UB.txt");
						command.add("LB.txt");
					} else {
						command.add("null");
						command.add("null");
					}
					
					ProcessBuilder builder = new ProcessBuilder(command);
					
					builder.directory(
														// working directory: in diesem wird nach den von dem Programm genuzten Files gesucht
							new File(FBASimVizAddon.pathExeAddons));
					
					if (computationTime > 0.9) {
						MainFrame.showMessageDialog(
															"<html>Please wait: the computation will take about " + computationTime + " minutes.", "Information");
					}
					
					status.setCurrentStatusText1("Simulation is running");
					status.setCurrentStatusText2("Please wait: the computation will take about " + computationTime + " minutes");
					status.setCurrentStatusValue(-1);
					
					final Process godot = builder.start();
					
					godot.getErrorStream().close();
					godot.getInputStream().close();
					godot.getOutputStream().close();
					
					int returnValue = godot.waitFor();
					status.setCurrentStatusValue(100);
					
					if (returnValue > 0)
						MainFrame.getInstance().showMessageDialog("Possible error. Return Value from simulator: " + returnValue);
					
					// gemappte Daten aus Graph entfernen
					for (GraphElement g : graph.getGraphElements())
						RemoveMappingDataAlgorithm.removeMappingDataFrom(g);
					
					// perform mapping
					Collection<GraphElement> targetGraphElements = new ArrayList<GraphElement>();
					targetGraphElements = graph.getGraphElements();
					
					String filename = par.filenameSBML;
					String filename2 = filename.replaceAll(".xml", "_");
					String filename3 = filename2.replaceAll(".sbml", "_");
					String fullName = par.path + filename3 + "DelitionFluxData.txt";
					String fullNameO = FBASimVizAddon.pathExeAddons + filename3 + "DelitionFluxData.txt";
					
					ExperimentInterface doc = XMLfile.getDocument(fullNameO, status);
					
					if (doc == null) {
						ErrorMsg.addErrorMessage("Experiment was null!");
					} else {
						Experiment2GraphHelper mappingService = new Experiment2GraphHelper();
						MapResult res = mappingService.mapDataToGraphElements(true, doc, targetGraphElements, null, false, GraffitiCharts.HIDDEN
																	.getName(), 0, false, false, false, false, false);
						
						FBAHelper.renameGraphAfterFluxSimulation(graph);
						
						MainFrame.showMessage("Data Mapping completed! (" + res.substanceCount + " FBA flux datasets)", MessageType.INFO);
						
						if (!isTesting) {
							// Aufrufen des SliderPanels
							GravistoService.getInstance().runAlgorithm(
																	new AnimatedFluxVisualizationDeletion(), getActionEvent());
						}
						// Resultfile wird in Modellordner verschoben
						FBAHelper.moveFile(fullNameO, fullName);
						// xml, VM/NM.txt UB./LB.txt werden gelöscht
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + par.filenameSBML);
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "VM.txt");
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "NM.txt");
						
						if (par.additionalConstraints == true) {
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "UB.txt");
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "LB.txt");
						}
						
					}
					
				} catch (Exception e) {
					ErrorMsg.addErrorMessage(e);
				}
				status.setCurrentStatusText2("Processing completed");
				
			}
		});
		// MainFrame.showMessage("Flux calculation for knock-out analysis, processing time "+(endO-startO)+" ms", MessageType.INFO);
	}// execute
	
	private String getDelReactionsString(Selection delReactionSelection) {
		String rct;
		String delReactions = "";
		int size = delReactionSelection.getNumberOfNodes();
		computationTime = Math.floor(noRcts * 0.45 / 60);
		if (computationTime == 0) {
			computationTime = 0.5;
		}
		int j = 0;
		for (GraphElement ge : delReactionSelection.getElements()) {
			j++;
			if (ge instanceof Node) {
				Node n = (Node) ge;
				rct = AttributeHelper.getLabel(n, "");
				if (j < size) {
					rct = rct.concat("_");
				}
				delReactions = delReactions.concat(rct);
			}
		}
		return delReactions;
	}
	
	// Eva: 9.11.
	private Boolean getDelCheck(Selection delReactionSelection, String ofReaction) {
		String rct;
		Boolean check = true;
		
		for (GraphElement ge : delReactionSelection.getElements()) {
			
			if (ge instanceof Node) {
				Node n = (Node) ge;
				rct = AttributeHelper.getLabel(n, "");
				if (rct.equals(ofReaction)) {
					check = false;
				}
				
			}
			
		}
		return check;
	}
	
	@Override
	protected String getImageFileName() {
		return "FBA1koSPEC.png";
	}
	
}