/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.calculation;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.TreeSet;

import org.ErrorMsg;
import org.graffiti.editor.GravistoService;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MessageType;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.BooleanParameter;
import org.graffiti.plugin.parameter.ObjectListParameter;
import org.graffiti.plugin.parameter.Parameter;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.editcomponents.chart_settings.GraffitiCharts;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.dbe.algorithms.RemoveMappingDataAlgorithm;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ExperimentInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.helper_classes.Experiment2GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.helper_classes.MapResult;
import de.ipk_gatersleben.ag_nw.graffiti.services.task.BackgroundTaskStatusProviderSupportingExternalCallImpl;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.ExecutionParameter;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBASimVizAddon;
import de.ipk_gatersleben.ag_pbi.fluxbalance.visualization.AnimatedFluxVisualizationFluxVariability;

/**
 * @author Eva
 */
public class StartCalculationAlgorithmFluxVariability extends AbstractStartCalculationAlgorithm {
	
	private static String ofReaction;
	
	private static boolean maximization = true;
	// private boolean minimization = false;
	// private final boolean additionalConstraints = true;
	// private boolean SBMLfileGeneration = true;
	private int ofCoef;
	private int noRcts;
	private double computationTime;
	
	public String getName() {
		return "<html><font color=\"#0101DF\">Flux Variability Analysis</font>";
	}
	
	@Override
	public String getDescription() {
		return "<html>" +
							"<b>With this command you may perform flux variability analysis.</b><br>" +
							"<br>" +
							"The resulting reaction-specific diagrams depict the minimal and maxi-<br>" +
							"mal flux values for a given optimal objective value as shown in the <br>" +
							"Example.<br>" +
							"<br>" +
							"To perform the analysis you have to specify:<br>" +
							"(1) the objective function (Reaction to be optimized)<br>" +
							"(2) the mode of optimization (Maximization, Minimization)<br>" +
							"<br>";
	}
	
	@Override
	public Parameter[] getParameters() {
		
		TreeSet<String> reactionNames = FBAHelper.getReactionNamesUnique(graph.getNodes());
		noRcts = reactionNames.size();
		computationTime = Math.floor(noRcts * 0.13 / 60);
		if (computationTime == 0) {
			computationTime = 0.5;
		}
		if (ofReaction == null)
			ofReaction = reactionNames.iterator().next();
		
		return new Parameter[] {
							new ObjectListParameter(reactionNames.contains(ofReaction) ? ofReaction : reactionNames.iterator().next(), "Reaction to be optimized",
									"Objective function", reactionNames),
							new BooleanParameter(true, "maximization", "Activated checkbox: maximization; not activated checkbox: minimization"),
		// new BooleanParameter(false, "Minimization", "Falls gew�hlt, wird die OF minimiert...OFcoef = 1"),
		// new BooleanParameter(false, "Account for changed reaction bounds", "Falls gew�hlt, werden die ge�nderten constraints ber�cksichtigt..."),
		// new BooleanParameter(false, "Automatic SBML-file generation", "Falls gewählt, wird Berechnung auf neu generiertem SBML-file durchgeführt"),
		
		};
	}
	
	@Override
	public void setParameters(Parameter[] params) {
		
		int i = 0;
		ofReaction = (String) ((ObjectListParameter) params[i++]).getValue();
		maximization = ((BooleanParameter) params[i++]).getBoolean();
		// this.minimization = ((BooleanParameter)params[i++]).getBoolean();
		if (maximization) {
			ofCoef = -1;
		} else {
			ofCoef = 1;
		}
		// this.additionalConstraints = ((BooleanParameter)params[i++]).getBoolean();
		// this.SBMLfileGeneration = ((BooleanParameter)params[i++]).getBoolean();
	}
	
	@Override
	public void check() throws PreconditionException {
		
		super.check();
		PreconditionException errors = new PreconditionException();
		
		checkForMatlabExecutable(errors, "VfluxVariabilityJan.exe");
		
		/*
		 * if (maximization == true && minimization == true){
		 * errors.add("Please chose one mode of optimization: maximization or minimization.");
		 * this.maximization = true;
		 * this.minimization = false;
		 * }
		 * if (maximization == false && minimization == false){
		 * errors.add("Please specify the mode of optimization: maximization or minimization.");
		 * this.maximization = true;
		 * this.minimization = false;
		 * }
		 */
		if (!errors.isEmpty()) {
			throw errors;
		}
	}
	
	public void execute() {
		
		final ExecutionParameter par = FBAHelper.prepareExecution(graph, false);
		if (par == null)
			return;
		
		doExecute(graph, par);
	}
	
	public void doExecute(final Graph graph, final ExecutionParameter par) {
		
		final BackgroundTaskStatusProviderSupportingExternalCallImpl status =
				new BackgroundTaskStatusProviderSupportingExternalCallImpl("Zeile 1", "Zeile 2");
		
		FBAHelper.runCalculation(status, getName(), new Runnable() {
			@Override
			public void run() {
				try {
					
					String SBMLmodel = par.filenameSBML;
					String OFrct = ofReaction;
					
								
					
					String OFcoef = (new Integer(ofCoef).toString());
					
					// exe-file und input/output files muessen im gleichen Ordener sein
					List<String> command = new ArrayList<String>();
					
					String pathExe = FBASimVizAddon.pathExeAddons + "VfluxVariabilityJan.exe";
					
					command.add(pathExe);
					command.add(SBMLmodel);
					command.add(OFrct);
					command.add(OFcoef);
					
					if (par.additionalConstraints == true) {
						//command.add("null");
						//command.add("null");
						command.add("UB.txt");
						command.add("LB.txt");
					} else {
						command.add("null");
						command.add("null");
					}
					
					ProcessBuilder builder = new ProcessBuilder(command);
					System.out.println(command);
					
					builder.directory(
														// working directory: in diesem wird nach den von dem Programm genuzten Files gesucht
							new File(FBASimVizAddon.pathExeAddons));
					
					if (computationTime > 0.9) {
						MainFrame.showMessageDialog(
								"<html>Please wait: the computation will take about " + computationTime + " minutes.", "Information");
					}
					
					status.setCurrentStatusText1("Simulation is running");
					status.setCurrentStatusText2("Please wait: the computation will take about " + computationTime + " minutes");
					status.setCurrentStatusValue(-1);
					
					final Process godot = builder.start();
					
					godot.getErrorStream().close();
					godot.getInputStream().close();
					godot.getOutputStream().close();
					
					int returnValue = godot.waitFor();
					status.setCurrentStatusValue(100);
					
					if (returnValue > 0) {
						MainFrame.getInstance().showMessageDialog("Possible error. Return Value from simulator: " + returnValue);
					}
					
					// gemappte Daten aus Graph entfernen
					for (GraphElement g : graph.getGraphElements())
						RemoveMappingDataAlgorithm.removeMappingDataFrom(g);
					
					// perform mapping
					Collection<GraphElement> targetGraphElements = new ArrayList<GraphElement>();
					targetGraphElements = graph.getGraphElements();
					
					String filename = par.filenameSBML;
					String filename2 = filename.replaceAll(".xml", "_");
					String filename3 = filename2.replaceAll(".sbml", "_");
					String fullName = par.path + filename3 + "FluxVariability.txt";
					String fullNameO = FBASimVizAddon.pathExeAddons + filename3 + "FluxVariability.txt";
					
					
					ExperimentInterface doc = XMLfile.getDocument(fullNameO, status);
					
					if (doc == null) {
						ErrorMsg.addErrorMessage("Experiment was null!");
					} else {
						Experiment2GraphHelper mappingService = new Experiment2GraphHelper();
						MapResult res = mappingService.mapDataToGraphElements(true, doc, targetGraphElements, null, false, GraffitiCharts.HIDDEN
																	.getName(), 0, false, false, false, false, false);
						
						FBAHelper.renameGraphAfterFluxSimulation(graph);
						
						MainFrame.showMessage("Data Mapping completed! (" + res.substanceCount + " FBA flux datasets)", MessageType.INFO);
						if (!isTesting) {
							// Aufrufen des SliderPanels
							GravistoService.getInstance().runAlgorithm(
																	new AnimatedFluxVisualizationFluxVariability(), getActionEvent());
						}
						// FBAHelper.copyFile(fullNameO, fullName);
						
						// Resultfile wird in Modellordner verschoben
						FBAHelper.moveFile(fullNameO, fullName);
						// xml, VM/NM.txt UB./LB.txt werden gelöscht
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + par.filenameSBML);
						
						if (par.additionalConstraints == true) {
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "UB.txt");
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "LB.txt");
						} else {
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "VM.txt");
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "NM.txt");
						}
						
					}
					
				} catch (Exception e) {
					ErrorMsg.addErrorMessage(e);
				}
				status.setCurrentStatusText2("Processing completed");
				
			}
		});
		// MainFrame.showMessage("Flux variability analysis, processing time "+(endO-startO)+" ms", MessageType.INFO);
	}// execute
	
	@Override
	protected String getImageFileName() {
		return "FBA1FV.png";
	}
	
}