/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.calculation;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.TreeSet;

import org.ErrorMsg;
import org.graffiti.editor.GravistoService;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MessageType;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.BooleanParameter;
import org.graffiti.plugin.parameter.ObjectListParameter;
import org.graffiti.plugin.parameter.Parameter;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.editcomponents.chart_settings.GraffitiCharts;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.dbe.algorithms.RemoveMappingDataAlgorithm;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ExperimentInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.helper_classes.Experiment2GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.layout_control.helper_classes.MapResult;
import de.ipk_gatersleben.ag_nw.graffiti.services.task.BackgroundTaskStatusProviderSupportingExternalCallImpl;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.ExecutionParameter;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBASimVizAddon;
import de.ipk_gatersleben.ag_pbi.fluxbalance.visualization.AnimatedFluxVisualizationSliderRobustness;

/**
 * @author Eva
 */
public class StartCalculationAlgorithmRobustnessNew extends AbstractStartCalculationAlgorithm {
	
	private static String ofReaction;
	private static boolean maximization = true;
	// private boolean minimization = false;
	private static boolean linearOpt = false;
	private String nlopt;
	private static String vpReaction;
	// private boolean additionalConstraints = true;
	// private boolean SBMLfileGeneration = true;
	private int ofCoef;
	
	public String getName() {
		return "<html>" +
							"<font color=\"#0101DF\">Robustness analysis (specified)</font>";
	}
	
	@Override
	public String getDescription() {
		return "<html>" +
							"<b>With this command you may perform robustness analysis<br>" +
							"based on a user-specified reaction.</b><br>" +
							"<br>" +
							"To perform the analysis you have to specify:<br>" +
							"(1) the objective function (Reaction to be optimized)<br>" +
							"(2) the mode of optimization (Maximization, Minimization)<br>" +
							"(3) the type of optimization (Linear, Non-linear)<br>" +
							"(4) the reaction to be varied in order to study the effect<br>" +
							"of its flux variation on the objective function<br>" +
							"<br>";
	}
	
	@Override
	public Parameter[] getParameters() {
		
		TreeSet<String> reactionNames = FBAHelper.getReactionNamesUnique(graph.getNodes());
		if (ofReaction == null)
			ofReaction = reactionNames.iterator().next();
		if (vpReaction == null)
			vpReaction = reactionNames.iterator().next();
		return new Parameter[] {
							new ObjectListParameter(reactionNames.contains(ofReaction) ? ofReaction : reactionNames.iterator().next(), "Reaction to be optimized",
									"Objective function", reactionNames),
							new BooleanParameter(maximization, "Maximization", "Activated checkbox: maximization; not activated checkbox: minimization"),
							new BooleanParameter(linearOpt, "Linear Optimization",
												"Activated checkbox: linear optimisation; not activated checkbox: nonlinear optimisation (2-step optimisation procedure)"),
							// new BooleanParameter(false, "Minimization", "Falls gew�hlt, wird die OF minimiert...OFcoef = 1"),
				new ObjectListParameter(reactionNames.contains(vpReaction) ? vpReaction : reactionNames.iterator().next(), "Reaction to be varied",
						"Reaction to be varied", reactionNames),
		// new BooleanParameter(false, "Account for changed reaction bounds", "Falls gew�hlt, werden die ge�nderten constraints ber�cksichtigt..."),
		// new BooleanParameter(false, "Automatic SBML-file generation", "Falls gewählt, wird Berechnung auf neu generiertem SBML-file durchgeführt"),
		
		};
	}
	
	@Override
	public void setParameters(Parameter[] params) {
		
		int i = 0;
		ofReaction = (String) ((ObjectListParameter) params[i++]).getValue();
		maximization = ((BooleanParameter) params[i++]).getBoolean();
		// this.minimization = ((BooleanParameter)params[i++]).getBoolean();
		if (maximization) {
			ofCoef = -1;
		} else {
			ofCoef = 1;
		}
		linearOpt = ((BooleanParameter) params[i++]).getBoolean();
		if (linearOpt) {
			nlopt = "false";
		} else {
			nlopt = "true";
		}
		vpReaction = (String) ((ObjectListParameter) params[i++]).getValue();
		// this.additionalConstraints = ((BooleanParameter)params[i++]).getBoolean();
		// this.SBMLfileGeneration = ((BooleanParameter)params[i++]).getBoolean();
	}
	
	@Override
	public void check() throws PreconditionException {
		super.check();
		PreconditionException errors = new PreconditionException();
		
		checkForMatlabExecutable(errors, "VrobSpecJan.exe");
		
		/*
		 * if (maximization == true && minimization == true){
		 * errors.add("Please chose one mode of optimization: maximization or minimization.");
		 * this.maximization = true;
		 * this.minimization = false;
		 * }
		 * if (maximization == false && minimization == false){
		 * errors.add("Please specify the mode of optimization: maximization or minimization.");
		 * this.maximization = true;
		 * this.minimization = false;
		 * }
		 */
		if (!errors.isEmpty()) {
			throw errors;
		}
	}
	
	public void execute() {
		
		final ExecutionParameter par = FBAHelper.prepareExecution(graph, true);
		if (par == null)
			return;
		
		doExecute(graph, par);
	}
	
	public void doExecute(final Graph graph, final ExecutionParameter par) {
		
		if (ofReaction != null && vpReaction != null && ofReaction.equals(vpReaction)) {
			MainFrame.showMessageDialog("The reaction to be optimized has to be different from the reaction to be varied.", "Execution Aborted");
			ofReaction = null;
			vpReaction = null;
			return;
		}
		
		final BackgroundTaskStatusProviderSupportingExternalCallImpl status =
				new BackgroundTaskStatusProviderSupportingExternalCallImpl("Zeile 1", "Zeile 2");
		
		FBAHelper.runCalculation(status, getName(), new Runnable() {
			@Override
			public void run() {
				try {
					
					String SBMLmodel = par.filenameSBML;
					String opt = nlopt;
					String OFrct = ofReaction;
					String OFcoef = (new Integer(ofCoef).toString());
					String VPrct = vpReaction;
					
					// exe-file und input/output files muessen im gleichen Ordener sein
					List<String> command = new ArrayList<String>();
					
					String pathExe = FBASimVizAddon.pathExeAddons + "VrobSpecJan.exe";
					command.add(pathExe);
					command.add(SBMLmodel);
					command.add(opt);
					command.add(OFrct);
					command.add(OFcoef);
					command.add(VPrct);
					command.add("NM.txt");
					command.add("VM.txt");
					
					if (par.additionalConstraints == true) {
						command.add("UB.txt");
						command.add("LB.txt");
					} else {
						command.add("null");
						command.add("null");
					}
					
					ProcessBuilder builder = new ProcessBuilder(command);
					
					builder.directory(
														// working directory: in diesem wird nach den von dem Programm genuzten Files gesucht
							new File(FBASimVizAddon.pathExeAddons));
					
					status.setCurrentStatusText1("Simulation is running");
					status.setCurrentStatusText2("Please wait: the computation will take about 10 seconds");
					status.setCurrentStatusValue(-1);
					
					final Process godot = builder.start();
					
					godot.getErrorStream().close();
					godot.getInputStream().close();
					godot.getOutputStream().close();
					
					int returnValue = godot.waitFor();
					status.setCurrentStatusValue(100);
					
					if (returnValue > 0) {
						MainFrame.getInstance().showMessageDialog("Possible error. Return Value from simulator: " + returnValue);
					}
					
					// gemappte Daten aus Graph entfernen
					for (GraphElement g : graph.getGraphElements())
						RemoveMappingDataAlgorithm.removeMappingDataFrom(g);
					
					// perform mapping
					Collection<GraphElement> targetGraphElements = new ArrayList<GraphElement>();
					targetGraphElements = graph.getGraphElements();
					
					String filename = par.filenameSBML;
					String filename2 = filename.replaceAll(".xml", "_");
					String filename3 = filename2.replaceAll(".sbml", "_");
					String fullName = par.path + filename3 + "RobustnessAnalysisSpec.txt";
					String fullNameO = FBASimVizAddon.pathExeAddons + filename3 + "RobustnessAnalysisSpec.txt";
					
					ExperimentInterface doc = XMLfile.getDocument(fullNameO, status);
					
					if (doc == null) {
						ErrorMsg.addErrorMessage("Experiment was null!");
					} else {
						Experiment2GraphHelper mappingService = new Experiment2GraphHelper();
						MapResult res = mappingService.mapDataToGraphElements(true, doc, targetGraphElements, null, false, GraffitiCharts.HIDDEN
																	.getName(), 0, false, false, false, false, false);
						
						FBAHelper.renameGraphAfterFluxSimulation(graph);
						
						MainFrame.showMessage("Data Mapping completed! (" + res.substanceCount + " fba flux datasets)", MessageType.INFO);
						if (!isTesting) {
							// Aufrufen des SliderPanels
							GravistoService.getInstance().runAlgorithm(
																	new AnimatedFluxVisualizationSliderRobustness(), getActionEvent());
						}
						// FBAHelper.copyFile(fullNameO, fullName);
						
						// Resultfile wird in Modellordner verschoben
						FBAHelper.moveFile(fullNameO, fullName);
						// xml, VM/NM.txt UB./LB.txt werden gelöscht
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + par.filenameSBML);
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "VM.txt");
						FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "NM.txt");
						
						if (par.additionalConstraints == true) {
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "UB.txt");
							FBAHelper.deleteFile(FBASimVizAddon.pathExeAddons + "LB.txt");
						}
						
					}
				} catch (Exception e) {
					ErrorMsg.addErrorMessage(e);
				}
				status.setCurrentStatusText2("Processing completed");
				
			}
		});
		// MainFrame.showMessage("Flux calculation for robustness analysis, processing time "+(endO-startO)+" ms", MessageType.INFO);
	}// execute
	
	@Override
	protected String getImageFileName() {
		return "FBA1ROspec.png";
	}
	
}