/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction;

import java.io.File;

import org.AttributeHelper;
import org.OpenFileDialogService;
import org.StringManipulationTools;
import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.Parameter;

import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.AbstractStartCalculationAlgorithm;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

/**
 * @author Eva
 */
public class CreateSBMLFile extends AbstractStartCalculationAlgorithm {
	
	public String getName() {
		return "<html>" +
							"<font color=\"#045FB4\">Create SBML file</font>";
	}
	
	@Override
	public String getDescription() {
		return "<html>" +
							"<b>With this command you may create a SBML file which <br>" +
							"forms the basis for subsequent model analysis.<br>";
		
	}
	
	@Override
	public Parameter[] getParameters() {
		
		return new Parameter[] {

		};
		
	}
	
	@Override
	public void setParameters(Parameter[] params) {
		
	}
	
	@Override
	public void check() throws PreconditionException {
		super.check();
		
		PreconditionException errors = new PreconditionException();
		
		checkForCorrectShapes(errors, graph);
		
		checkForUnconnectedNodes(errors, graph);
		
		// check for forbidden html-tags in the labels
		for (Node n : graph.getNodes()) {
			String lbl = AttributeHelper.getLabel(n, "");
			if (lbl.contains("<") || lbl.contains(">")) // TODO? also not valid sbml: comma, space, plus, minus, semikolon
				errors.add("Node label \"" + StringManipulationTools.removeHTMLtags(lbl)
						+ "\" contains forbidden \"&lt;\" or \"&gt;\" tags.");
			if (lbl.length() > 0 && FBAHelper.isReaction(n) && Character.isDigit(lbl.charAt(0)))
				errors.add("Reaction label \"" + lbl + "\" must not start with a number.");
			if (FBAHelper.isReaction(n)) {
				// one reaction is reversible and irreversible at the same time -> number of arrowheads are different
				int nbrarrows = -1;
				for (Edge ed : n.getEdges())
					if (nbrarrows == -1)
						nbrarrows = FBAHelper.getNbrOfHeads(ed);
					else
						if (nbrarrows != FBAHelper.getNbrOfHeads(ed))
							errors.add("<html>It is unclear, if the reaction \"" + AttributeHelper.getLabel(n, null) + "\" is reversible or not.<br>" +
													"All adjacent edges of a reaction node have to have the same number of arrowheads.");
			}
		}
		
		if (!errors.isEmpty()) {
			throw errors;
		}
	}
	
	public void execute() {
		new SBMLfile(graph);
		MainFrame.showMessageDialog("SBML file created", "Information");
	}
	
	@Override
	protected String getImageFileName() {
		return null;
	}
	
	public static File askUserForSBMLFile(File activeDir) {
		// Einlesen des SBML-files (*.xml)
		if (activeDir != null)
			OpenFileDialogService.setActiveDirectoryFrom(activeDir);
		File sbmlf = OpenFileDialogService.getFile(
				new String[] { ".xml" }, "SBML-file (*.xml)", "Select SBML File");
		if (sbmlf == null) {
			MainFrame.showMessageDialog("Please select an SBML-file (*.xml)!", "Error");
			return null;
		} else
			return sbmlf;
	}
	
}