/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.TreeSet;

import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;

import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.NodeHelper;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

public class SBMLfile {
	
	String fileName = null;
	boolean erroroccured = false;
	
	public SBMLfile(Graph graph) {
		try {
			
			new AppendSBMLAttributes(graph);
			
			// SBML-File erzeugen
			// create SBMLfile-Name
			String absPathfileName = getSBMLFilenameFromGraph(graph);
			
			// modelID
			String modelID = graph.getName();
			String modelID2 = modelID.replaceAll(".gml", "");
			
			ArrayList<?> listOfReactions = new ArrayList<Object>();
			listOfReactions = getReactionList(graph.getNodes());
			
			TreeSet<String> listOfMetabolites = new TreeSet<String>();
			listOfMetabolites = FBAHelper.getMetaboliteNamesUnique(graph.getNodes());
			
			// create SBMLfile
			createSBMLfile(absPathfileName, modelID2, listOfReactions, listOfMetabolites);
			
			fileName = absPathfileName;
			
		} catch (final Exception err) {
			ErrorMsg.addErrorMessage(err);
			erroroccured = true;
		}
	}
	
	public static String getSBMLFilenameFromGraph(Graph graph) {
		return graph.getName(true).replaceAll(".gml", ".xml");
	}
	
	String getName() {
		return fileName;
	}
	
	public void createSBMLfile(String absPathfileName2, String modelID2, ArrayList<?> listOfReactions, TreeSet<String> listOfMetabolites) {
		
		try {
			BufferedWriter SBMLfile = new BufferedWriter(new FileWriter(absPathfileName2));
			SBMLfile.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			SBMLfile.newLine();
			SBMLfile.write("<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">");
			SBMLfile.newLine();
			SBMLfile.write(getTab(1) + "<model id=\"" + modelID2 + "\" name=\"" + modelID2 + "\">");
			SBMLfile.newLine();
			SBMLfile.write(getTab(2) + "<listOfCompartments>");
			SBMLfile.newLine();
			SBMLfile.write(getTab(3) + "<compartment id=\"Internal_Species\"/>");
			SBMLfile.newLine();
			SBMLfile.write(getTab(2) + "</listOfCompartments>");
			SBMLfile.newLine();
			SBMLfile.write(getTab(2) + "<listOfSpecies>");
			SBMLfile.newLine();
			Iterator<String> iterM = listOfMetabolites.iterator();
			while (iterM.hasNext()) {
				String metabol = iterM.next();
				// System.out.println("Metabolite: "+metabol);
				SBMLfile.write(getTab(3) + "<species id=\"" + metabol + "\" name=\"" + metabol
									+ "\" boundaryCondition=\"false\" compartment=\"Internal_Species\"/>");
				SBMLfile.newLine();
			}
			SBMLfile.write(getTab(2) + "</listOfSpecies>");
			SBMLfile.newLine();
			SBMLfile.write(getTab(2) + "<listOfReactions>");
			SBMLfile.newLine();
			
			Iterator<?> iterR = listOfReactions.iterator();
			while (iterR.hasNext()) {
				String rct = (String) iterR.next();
				SBMLfile.write(rct);
				
			}
			SBMLfile.write(getTab(2) + "</listOfReactions>");
			SBMLfile.newLine();
			SBMLfile.write(getTab(1) + "</model>");
			SBMLfile.newLine();
			SBMLfile.write("</sbml>");
			SBMLfile.close();
			
		} catch (IOException e1) {
			ErrorMsg.addErrorMessage(e1);
		}
	}
	
	// public static TreeSet<String> getReactionNames(Collection<Node> nodes) {
	//		
	// TreeSet<String> reactions = new TreeSet<String>();
	// for (Node n : nodes) {
	// NodeHelper node = new NodeHelper(n);
	// String label = AttributeHelper.getLabel(n, "");
	//			
	// if (label != null) {
	// if (FBAHelper.isReaction(n)) {
	// String sbmlID = org.AttributeHelper.getSBMLid(node);
	// reactions.add(sbmlID);
	// }
	// }
	// }
	// return reactions;
	// }
	//	
//	public static TreeSet<String> getSBMLIDMetaboliteNames(Collection<Node> nodes) {
//		
//		TreeSet<String> metabolites = new TreeSet<String>();
//		for (Node n : nodes) {
//			NodeHelper node = new NodeHelper(n);
//			String label = AttributeHelper.getLabel(n, "");
//			
//			if (label != null) {
//				if (AttributeHelper.getShape(n).contains("circle")) {
//					String sbmlID = org.AttributeHelper.getSBMLid(node);
//					
//					//Alternativ:
//					String sbmlID = label;
//					
//					
//					metabolites.add(sbmlID);
//				}
//			}
//		}
//		return metabolites;
//	}
	
	public static String getTab(int d) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < d; i++)
			sb.append("\t");
		return sb.toString();
	}
	
	public static String createSBMLString(String rctLabel, String rev, ArrayList<String> reactants, ArrayList<String> products) {
		String SBMLString = "";
		String append1 = getTab(3) + "<reaction id=\"" + rctLabel + "\" reversible=\"" + rev + "\">" + "\n";
		String append2 = getTab(4) + "<listOfReactants>" + "\n";
		String append3 = "";
		
		Iterator<String> iter = reactants.iterator();
		while (iter.hasNext()) {
			String reactantLabel = iter.next();
			String reactantStoichcoef = iter.next();
			String append4 = getTab(5) + "<speciesReference species=\"" + reactantLabel + "\" stoichiometry=\"" + reactantStoichcoef + "\"/>" + "\n";
			append3 = append3 + append4;
		}
		
		String append5 = getTab(4) + "</listOfReactants>" + "\n";
		String append6 = getTab(4) + "<listOfProducts>" + "\n";
		String append7 = "";
		
		Iterator<String> iterP = products.iterator();
		while (iterP.hasNext()) {
			String productLabel = iterP.next();
			String productStoichcoef = iterP.next();
			String append8 = getTab(5) + "<speciesReference species=\"" + productLabel + "\" stoichiometry=\"" + productStoichcoef + "\"/>" + "\n";
			append7 = append7 + append8;
		}
		String append9 = getTab(4) + "</listOfProducts>" + "\n";
		String append10 = getTab(3) + "</reaction>" + "\n";
		
		SBMLString = append1 + append2 + append3 + append5 + append6 + append7 + append9 + append10;
		return SBMLString;
	}
	
	public static ArrayList<String> getReactionList(Collection<Node> nodes) {
		
		String rev = "";
		ArrayList<String> listOfRcts = new ArrayList<String>();
		for (Node n : nodes) {
			if (FBAHelper.isReaction(n)) {
				String rctLabel = AttributeHelper.getLabel(n, "");
				if (AttributeHelper.isSBMLreversibleReaction(n)) {
					rev = "true";
				} else {
					rev = "false";
				}
				ArrayList<String> reactants = new ArrayList<String>();
				ArrayList<String> products = new ArrayList<String>();
				
				for (Edge inEdge : n.getAllInEdges()) {
					Node nS = inEdge.getSource();
					if (nS != null) {
						String reactantLabel = AttributeHelper.getLabel(nS, "");
						reactants.add(reactantLabel);
						String stoichCoefR = AttributeHelper.getLabel(inEdge, "");
						if (stoichCoefR.equals("")) {
							stoichCoefR = "1";
						}
						
						// Hinter Reactantname wird direkt SToichCoefficient eingelesen
						reactants.add(stoichCoefR);
					}
				}
				
				for (Edge outEdge : n.getAllOutEdges()) {
					Node nT = outEdge.getTarget();
					if (nT != null) {
						String productLabel = AttributeHelper.getLabel(nT, "");
						products.add(productLabel);
						String stoichCoefP = AttributeHelper.getLabel(outEdge, "");
						if (stoichCoefP.equals("")) {
							stoichCoefP = "1";
						}
						products.add(stoichCoefP);
					}
				}
				
				String SBMLString = createSBMLString(rctLabel, rev, reactants, products);
				listOfRcts.add(SBMLString);
			}
		}
		return listOfRcts;
	}
}
