package de.ipk_gatersleben.ag_pbi.fluxbalance.startsim;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.TreeSet;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;

import org.AttributeHelper;
import org.FolderPanel;
import org.StringManipulationTools;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.AbstractAlgorithm;
import org.graffiti.plugin.algorithm.AlgorithmWithComponentDescription;
import org.graffiti.plugin.algorithm.PreconditionException;

public abstract class AbstractStartCalculationAlgorithm extends AbstractAlgorithm implements AlgorithmWithComponentDescription {
	
	protected boolean isTesting = false;
	
	public AbstractStartCalculationAlgorithm() {
		this(false);
		
	}
	
	public AbstractStartCalculationAlgorithm(boolean isTesting) {
		super();
		this.isTesting = isTesting;
		
	}
	
	@Override
	public String getCategory() {
		return "FBA";
	}
	
	protected abstract String getImageFileName();
	
	public JComponent getDescriptionComponent() {
		if (getImageFileName() == null)
			return null;
		ClassLoader cl = FBASimVizAddon.class.getClassLoader();
		String path = FBASimVizAddon.class.getPackage().getName().replace('.', '/');
		
		// ImageFolder in bin/de/ipk_gatersleben/ag_pbi/fluxbalance/startsim
		java.net.URL imageURL = cl.getResource(path + "/images/" + getImageFileName());
		
		if (imageURL != null) {
			ImageIcon icon = new ImageIcon(imageURL);
			return FolderPanel.getBorderedComponent(new JLabel(icon), 3, 3, 8, 8);
		} else {
			System.err.println("Couldn't find file: " + path);
			return FolderPanel.getBorderedComponent(new JLabel("Imagedescription not found"), 5, 5, 5, 5);
		}
	}
	
	@Override
	public void check() throws PreconditionException {
		PreconditionException errors = new PreconditionException();
		if (graph == null || graph.getNumberOfNodes() < 3) {
			errors.add("No active graph or graph too small (less than 3 nodes)!");
		}
		if (graph != null) {
			TreeSet<String> reactionNames = FBAHelper.getReactionNamesUnique(graph.getNodes());
			if (reactionNames.size() < 2)
				errors.add("Graph needs to contain at least 2 reactions!");
		}
		
		HashSet<String> label = new HashSet<String>();
		for (Node n : graph.getNodes()) {
			if (FBAHelper.isReaction(n)) {
				String l = AttributeHelper.getLabel(n, "");
				if (label.contains(l))
					errors.add("Label " + l + " is used more than once");
				label.add(l);
			}
		}
		
		/*
		 * checkForCorrectShapes(errors,graph);
		 * checkForUnceonnectedNodes(errors,graph);
		 * //label ohne Kanten
		 * for (Edge e : graph.getEdges()) {
		 * String labelE= AttributeHelper.getLabel(e, "1");
		 * System.out.println("Edge-Label: "+labelE);
		 * if (AttributeHelper.getLabel(e, "").equals("")){
		 * errors.add("Missing edge label, please amend!");
		 * }
		 * }
		 */
		if (!errors.isEmpty())
			throw errors;
	}
	
	protected void checkForMatlabExecutable(PreconditionException errors, String fname) {
		String pathExeC = FBASimVizAddon.pathExeAddons + fname;
		if (!new File(pathExeC).exists())
			errors.add("The Matlab executables cannot be accessed. Please add the executable in the addon folder.");
	}
	
	protected void checkForCorrectShapes(PreconditionException errors, Graph graph) {
		// Graph-check:
		// korrekte Enzyme/Metabolit-Knotentypen
		ArrayList<String> metaboliteNames = new ArrayList<String>();
		if (graph != null) {
			metaboliteNames = FBAHelper.getMetaboliteNames(graph.getNodes());
			int rctNo = FBAHelper.getReactionNamesUnique(graph.getNodes()).size();
			int metNo = metaboliteNames.size();
			
			int nodesNo = FBAHelper.getNumberOfNodes(graph.getNodes());
			if (((rctNo + metNo) - nodesNo) != 0)
				errors.add("Please check elements of network map: metabolites have to be represented as cirlces, reactions have to be represented as rectangles.");
		}
	}
	
	protected void checkForUnconnectedNodes(PreconditionException errors, Graph graph) {
		// Single metabolites/enzymes
		List<String> singleNodes = null;
		if (graph != null) {
			singleNodes = FBAHelper.getSingleNodes(graph.getNodes());
			
			if (singleNodes.size() > 0) {
				String name1 = "";
				for (String n : singleNodes) {
					String name2 = n;
					name1 = name1 + name2 + ", ";
				}
				errors.add(StringManipulationTools.getWordWrap("The network map contains unconnected reaction and/or metabolite nodes: " + name1, 50));
			}
		}
	}
	
}
