/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/

package de.ipk_gatersleben.ag_pbi.fluxbalance.startsim;

import info.clearthought.layout.TableLayout;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.ErrorMsg;
import org.ReleaseInfo;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MyScrollLabel;
import org.graffiti.plugin.GenericPluginAdapter;
import org.graffiti.plugin.algorithm.Algorithm;

import de.ipk_gatersleben.ag_nw.graffiti.JLabelHTMLlink;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.addons.AddonAdapter;
import de.ipk_gatersleben.ag_pbi.fluxbalance.calculation.ConnectMetabolites;
import de.ipk_gatersleben.ag_pbi.fluxbalance.calculation.StartCalculationSelection;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.ModelReconstructionSelection;
import de.ipk_gatersleben.ag_pbi.fluxbalance.visualization.StartVisualizationSelection;

/**
 * @author Eva
 */
public class FBASimVizAddon extends AddonAdapter {
	
	/*
	 * TODO:
	 * 
	 * ideas:
	 * - cmatrix should allow more than to four metabolites for a reaction
	 * - why cant edge-labels contain double-numbers (why is no double coefficient allowed)?
	 * - should the sbml-export be valid sbml (comma, plus, spaces, ... should be removed from speciesname)
	 * - XMLFile is basically a copy of some EexperimentLoader-methods for loading csv-data files. couldnt we just use these?
	 * - if the Matlab executeables produce errors, no result file will be written and the user doesnt know what happened;
	 * maybe it could produce some kind of error-file instead
	 * 
	 * bugs:
	 * - simulator writes strange result files for fba with two sliders and example_network -> test
	 * - unittests for knockout, fluxvariability, .... -> evtl
	 * - fluxvariability does not work with example-network -> ANJA FRAGEN
	 * - SBMLFile.getSBMLIDMetaboliteNames -> here with sbml-id, but everywhere with label?
	 * -> OK, da SBML-id = label, man könnte auch alternativ getLabel nehmen (String sbmlID = label);-> gleiches Ergebnis
	 * - where is the Matlab sourcecode?
	 */

	public static boolean ready = false;
	public static boolean matlabInstalled = false;
	public static String pathExeAddons;
	
	@Override
	protected void initializeAddon() {
		pathExeAddons = ReleaseInfo.getAppSubdirFolderWithFinalSep("addons", "FBASimVis");
		String error = MATLABRuntimeInstalled();
		if (error == null) {
			ready = false;
			this.algorithms = new Algorithm[] {
								new ModelReconstructionSelection(),
								new StartCalculationSelection(),
								new StartVisualizationSelection(),
								new ConnectMetabolites(),
								// new TestEdgeColorSettingalg()
								};
			copyFilesToFBADir();
			matlabInstalled = true;
		} else
			informUserAboutMissingMatlabRuntime(error);
	}
	
	private void informUserAboutMissingMatlabRuntime(final String error) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JPanel message = new JPanel();
				message.setLayout(TableLayout.getLayout(TableLayout.FILL,
						new double[] { TableLayout.PREFERRED, TableLayout.PREFERRED, 10, TableLayout.PREFERRED, TableLayout.PREFERRED }));
				JLabel text = new JLabel("<html>" +
									"The Add-on \"FBASimVis\" needs a MATLAB Component<br>" +
									"Runtime (MCR) in order to simulate fluxes. Please<br>" +
									"follow the instructions found at:<br>&nbsp;");
				
				JLabelHTMLlink link = new JLabelHTMLlink("http://fbasimvis.ipk-gatersleben.de/installation.html",
						"http://fbasimvis.ipk-gatersleben.de/installation.html");
				message.add(text, "0,0");
				message.add(link, "0,1");
				
				message.add(new JLabel("Here is the error log:"), "0,3");
				message.add(new MyScrollLabel(error, 300, 100), "0,4");
				message.validate();
				JOptionPane.showMessageDialog(MainFrame.getInstance(), message, "Add-on deactivated!", JOptionPane.ERROR_MESSAGE);
			}
		});
	}
	
	public static void copyFilesToFBADir() {
		Runnable copyT = new Runnable() {
			public void run() {
				if (saveFiles(new String[] {
									"FBAmultiJan.ctf", "FBAmultiJan.exe", "libsbml.dll",
									"libexpat.dll", "VdelCompleteJan.ctf", "VdelCompleteJan.exe",
									"VdelSpecJan.ctf", "VdelSpecJan.exe",
									"VfluxVariabilityJan.ctf",
									"VfluxVariabilityJan.exe", "VrobCompleteJan.ctf",
									"VrobCompleteJan.exe",
									"VrobSpecJan.ctf", "VrobSpecJan.exe" }))
					ready = true;
			}
		};
		
		Thread t = new Thread(copyT);
		t.setName("Copy FBA-SimVis Executables");
		t.start();
	}
	
	private String MATLABRuntimeInstalled() {
		// Ueberpruefung, ob MATLAB runtime installiert ist: Test.exe erzeugt test.txt datei, falls diese nicht vorhanden ist, fehlt MCR
		try {
			if (saveFiles(new String[] { "test.exe", "test.ctf" })) {
				
				List<String> command = new ArrayList<String>();
				String pathExe = pathExeAddons + "test.exe";
				command.add(pathExe);
				
				ProcessBuilder builder = new ProcessBuilder(command);
				// working directory: in diesem wird nach den von dem Programm genuzten Files gesucht
				builder.directory(new File(pathExeAddons));
				
				final Process godot = builder.start();
				
				godot.getErrorStream().close();
				godot.getInputStream().close();
				godot.getOutputStream().close();
				
				String testfile = pathExeAddons + "test.txt";
				File f1 = new File(testfile);
				
				if (f1.exists()) {
					FBAHelper.deleteFile(pathExeAddons + "test.txt");
					return null;
				} else
					return "File " + testfile
							+ " was not written. this indicates, that the script didn't work and Matlab Runtime Component is not properly installed.";
			} else
				return "Files \"test.exe\" and \"test.ctf\" could not be copied to " + pathExeAddons;
		} catch (Exception e) {
			return e.getMessage();
		}
	}
	
	private static boolean saveFiles(String[] files) {
		boolean result = true;
		for (String file : files) {
			String src = "/exe/" + file;
			String tgt = pathExeAddons + file;
			// exePath: ReleaseInfo.getAppSubdirFolderWithFinalSep("addons", "fbaSimVis")+"meinSimulator.exe";
			// momentan werden bei jedem NEustart alle exe/ctfs neu kopiert, um bei neuer Version Problem bei gleicher exe-Bezeichnung zu umgehen
			boolean copyAllways = true;
			if (copyAllways || !new File(tgt).exists())
				if (!copyFile(src, tgt))
					result = false;
		}
		return result;
	}
	
	private static boolean copyFile(String src, String tgt) {
		try {
			System.out.print("Creating file " + tgt + "...");
			
			ClassLoader cl = FBASimVizAddon.class.getClassLoader();
			String path = FBASimVizAddon.class.getPackage().getName().replace('.', '/');
			InputStream inpS = cl.getResourceAsStream(path + src);
			InputStream in = inpS;
			FileOutputStream out;
			if (inpS != null)
				out = new FileOutputStream(tgt);
			else
				out = null;
			
			byte[] buf = new byte[32768];
			int i = 0;
			while ((i = inpS.read(buf)) != -1) {
				out.write(buf, 0, i);
			}
			
			in.close();
			out.close();
			// MainFrame.showMessage("File "+src+" saved as "+tgt, MessageType.INFO);
			System.out.println("OK");
			return true;
		} catch (Exception err) {
			ErrorMsg.addErrorMessage(err);
			return false;
		}
	}
	
	@Override
	public ImageIcon getIcon() {
		return getFBAIcon();
	}
	
	public static ImageIcon getFBAIcon() {
		try {
			ClassLoader cl = FBASimVizAddon.class.getClassLoader();
			String path = FBASimVizAddon.class.getPackage().getName().replace('.', '/');
			ImageIcon icon = new ImageIcon(cl.getResource(path + "/" + "images/fbalogo.png"));
			if (icon != null)
				return icon;
			else
				return GenericPluginAdapter.getAddonIcon();
		} catch (Exception e) {
			ErrorMsg.addErrorMessage(e);
			return GenericPluginAdapter.getAddonIcon();
		}
	}
	
}
