package de.ipk_gatersleben.ag_pbi.fluxbalance.unittests;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;

import org.graffiti.graph.AdjListGraph;
import org.graffiti.graph.Graph;
import org.graffiti.plugin.io.InputSerializer;
import org.graffiti.plugins.ios.importers.gml.GMLReader;
import org.graffiti.selection.Selection;
import org.junit.Assert;
import org.junit.Test;

import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.CreateCmatrix;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.CreateMetaboliteFile;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.CreateReactionFile;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.CreateSBMLFile;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.Reactionfile;
import de.ipk_gatersleben.ag_pbi.fluxbalance.reconstruction.SBMLfile;

public class TestReconstruction {
	
	@Test
	public void createSBMLFiles() throws Exception {
		int errcnt = 0;
		for (Graph g : getGraphs()) {
			File f = new File(SBMLfile.getSBMLFilenameFromGraph(g));
			try {
				f.delete();
				CreateSBMLFile alg = new CreateSBMLFile();
				alg.attach(g, new Selection("none"));
				alg.check();
				alg.execute();
				alg.reset();
			} catch (Exception e) {
				System.err.println("generating sbml failed for " + g.getName());
				errcnt++;
			}
			Assert.assertTrue(f.length() > 10);
			
		}
		
		Assert.assertEquals(0, errcnt);
		
	}
	
	@Test
	public void createMetaboliteFiles() throws Exception {
		int errcnt = 0;
		for (Graph g : getGraphs()) {
			File f = new File(CreateMetaboliteFile.getMetaboliteFilenameFromGraph(g));
			try {
				f.delete();
				CreateMetaboliteFile alg = new CreateMetaboliteFile(false);
				alg.attach(g, new Selection("none"));
				alg.check();
				alg.execute();
				alg.reset();
			} catch (Exception e) {
				System.err.println("generating metabolitefile failed for " + g.getName());
				errcnt++;
			}
			Assert.assertTrue(f.length() > 10);
		}
		
		Assert.assertEquals(0, errcnt);
		
	}
	
	@Test
	public void createCmatrixFiles() throws Exception {
		int errcnt = 0;
		for (Graph g : getGraphs()) {
			File f = new File(CreateCmatrix.getMatrixFilenameFromGraph(g));
			try {
				f.delete();
				ArrayList<String> list = CreateCmatrix.doCreateMatrix(new File(CreateMetaboliteFile.getMetaboliteFilenameFromGraph(g)), g);
				Assert.assertNotNull(list);
				// if there are transporteres in the graph, there will be unbalanced reactions anyway
				// Assert.assertTrue(list.size() == 0);
			} catch (Exception e) {
				System.err.println("generating cmatrix failed for " + g.getName());
				errcnt++;
			}
			Assert.assertTrue(f.length() > 10);
			
		}
		
		Assert.assertEquals(0, errcnt);
		
	}
	
	@Test
	public void createReactionFiles() throws Exception {
		int errcnt = 0;
		for (Graph g : getGraphs()) {
			File f = new File(Reactionfile.getReactionFilenameFromGraph(g));
			try {
				f.delete();
				CreateReactionFile alg = new CreateReactionFile();
				alg.attach(g, new Selection("none"));
				alg.check();
				alg.execute();
				alg.reset();
			} catch (Exception e) {
				System.err.println("generating reactionfile failed for " + g.getName());
				errcnt++;
			}
			Assert.assertTrue(f.length() > 10);
		}
		
		Assert.assertEquals(0, errcnt);
		
	}
	
	public static ArrayList<Graph> getGraphs() throws Exception {// int randomGraphs) throws Exception {
		InputSerializer is = new GMLReader();
		
		ArrayList<Graph> list = new ArrayList<Graph>();
		FilenameFilter ff = new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				return name.endsWith(".gml");
			}
		};
		File dir = new File(TestReconstruction.class.getResource("graphs/C4_core.gml").toURI()).getParentFile();
		for (File f : dir.listFiles(ff)) {
			Graph g = new AdjListGraph();
			is.read(f.getAbsolutePath(), g);
			if (g != null && (g.getNumberOfEdges() + g.getNumberOfNodes()) > 0) {
				list.add(g);
				g.setName(f.getAbsolutePath());
			}
		}
		
		// for (int i = 0; i < randomGraphs; i++) {
		// Graph g = null;
		// if (Math.random() < 0.5)
		// g = ErdosRenyiGraphGenerator.createGraph((int) Math.random() * 200, Math.random() < 0.5, Math.random(), Math.random() < 0.5, Math.random() < 0.5);
		// else
		// g = WattsStrogatzGraphGenerator.createGraph((int) Math.random() * 200, Math.random() < 0.5, (int) Math.random() * 5, Math.random());
		// g.setName(TemporaryFilesOfUnittests.getPath() + "/" + getRandomName() + ".gml");
		// }
		
		return list;
	}
	
	// private static String getRandomName() {
	// Random random = new Random();
	// int len = random.nextInt(20);
	// StringBuilder sb = new StringBuilder();
	// while (len > 0) {
	// len--;
	// sb.append(getRandomChar(random));
	// }
	// return sb.toString();
	// }
	//	
	// private static Character getRandomChar(Random random) {
	// int c;
	// boolean simple = true;
	// if (simple)
	// c = (random.nextInt(90 - 65) + 65);
	// else
	// c = random.nextInt(65000);
	// return new Character((char) c);
	// }
	
}
